clear
clc

%% For simplicity, we use a Gauss-Sciedel procedure and iteratire
%% across the parameters. This code is setup to only do one iteation from the starting values but this should only
%% affect the MC variability of the estimator. More iterations should be done to make the MC variance smaller


rng(1234)

T = 500; % Sample size
R = 1000; % Number of replications for MC

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%% Setting parameters for the SV process %%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%rho=0;
%% Setting one
k=.9;
sigmav=.36;
d=-.74;

%% Setting two
% k=.98;
% sigmav=.06;
% d=-.14;


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
M=10; % Number of Monte Carlo samples used in the I-I procedure

%% Fixing what auxiliary model to use. In the paper we just use the GARCH(1,1) with normal errors.  
switch_var = 'normal';
p = 1;
q = 1;
m = 1;
tic
for i=1:1
 % Generating data for the experiment    
[ r, x ] = gen_SVAR_New(k,sigmav,d,T);
r = r';

% Estimating the constrained auxiliary parameters
[beta_hat, space1, space2, InvHess, space4, scores1, grad1, Hess] = garchpq1(r , p , q);

BETA_hat(:,i) = beta_hat;
% Estimating the unconstrained auxiliary parameters
beta_hat_hat = beta_hat - InvHess*sum(scores1,1)';

BETA_hat_hat(:,i) = beta_hat_hat;

%% No weighting matrix needed as we have exact identification here. 
I0 = eye(3);
% Generating the random numbers needed for I-I
nu1 = randn(T*M+10000,1);
nu2 = chi2rnd(1,1,T*M+10000);
nu2=log(nu2)+1.2749;
nu2 =nu2';

% Passing array entires to be used in the optimization of the obj. function. 
varInf = cell(1,3);
varInf{1,1}=[nu1 nu2];
varInf{1,2}=beta_hat_hat;
varInf{1,3} = switch_var;
varInf{1,4} = beta_hat;
varInf{1,5} = inv(I0);
varInf{1,6} = beta_hat_hat-beta_hat;
varInf{1,7}=T;
varInf{1,8} = mean(r);

%%%%%%%%%%%%%%%%%%%%% First for kappa %%%%%%%%%%%%%%%%%%%%%%%%%%%%%
varInf{1,9} = 'one'; % Which part of the algorithm we're in; i.e., first step or second. 
varInf{1,10} = .36; % Fixing the parameter we are not iterative over for now
param2 = .5:.001:1; % Optimization will be carried out using unconstrained nonlinear
% minimization using a mixof grid search + Simplex method. Therefore, we need to first specify a coarse grid. 

gdvalues = {param2};
objfun = 'obj_funII_NORMAL';
addarg = varInf; % passing the strucutre to the function

% Here is the optimization for the kappa component 
[k_hat,val] = simpgdsearch(objfun,gdvalues,addarg);
 
 %%%%%%%%%%%%%%%%%%%%% Then for Sigma %%%%%%%%%%%%%%%%%%%%%%%%%%%%%
varInf{1,9} = 'two';
param2 = linspace(0,1,1000); % Optimization will be carried out using unconstrained nonlinear
% minimization using a mixof grid search + Simplex method. Therefore, we need to first specify a coarse grid. 
param2(8) = .36; % starting value for sigma
varInf{1,10} = k_hat; % Starting value for the kappa

gdvalues = {param2};
objfun = 'obj_funII_NORMAL';
addarg = varInf;

% Here is the optimization for the sigma component 
[sig_hat,val] = simpgdsearch(objfun,gdvalues,addarg);

% In this parameterization, we can estimate omega simply from the mean of
% returns. For further details, see the generation of the data in
% gen_SVAR_New.m. 
THETA(i,:) = [mean(r);k_hat;sig_hat];

end
toc
