
clear
clc
rng(1234)


load('spx_OxMan.mat') % Loading the oxford man data


data = spxOxMan1(4268:4768,:); % Taking two years  of data starting in 2017. 
data = table2array(data(:,4:end)); % transforming from array

rt_day = data(:,18); % Pulling data column
rt_day = (rt_day - mean(rt_day)); % de-meaning returns
rvt = data(:,11); % realized vol column
bvt = data(:,7); % bi-power variation
jt = (rvt-bvt).*((rvt-bvt)>0); % realized jump measure



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Specifying Error Distribution for GARCH % This code if from Sheppard 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


error_type = 'STUDENTST';
tarch_type = 2;

 if error_type == 'STUDENTST'
    errors=2;
 else
     errors=1;
 end
 
 % Tarch orders
p = 1;
o = 0;
q = 1;

% Auxiliary model estimation: Tarch
 [parameters1, LL, ht, VCVrobust, VCV, scores, diagnostics] = tarch(rt_day, p, o, q, error_type,2,[.0000001;.01;.98;5]);

% Calculating the FUNC estimator
FUNC = parameters1 - VCV*mean(scores)';
% Calculating additional auxiliary statistics 
eta1_y = mean(sign(rt_day).*sqrt(jt));
eta2_y = mean((jt-mean(jt)).^2);
eta3_y = mean((jt-mean(jt)).*[0;(jt(1:end-1)-mean(jt))]);
Eta_y = [eta1_y;eta2_y;eta3_y]; 

M=4680; % number of 5 second intervals in a day
T=length(data)+1;
H=10; nrepl = H;% Number of II reps.
D=12*5; % sampling frequency at 5 minutes (12*5*5 seconds)
beta_hat = parameters1; % auxiliary parameters
W=randn(T*M,6,H);  % big array with the random components that are fixed within the II loop


 fun = @(t)IIFun(t,1, 0, 1, W, beta_hat,FUNC, Eta_y,T,M,H,D);
 
 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Variance parameters of SDE: Starting values
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Estimation can be carried out several ways. The objective function is
% fairly flat in certain regions, so a large grid search is the best way to
% to this. We do this in a stochastic manner, although other approaches
% could also be used. This yields something that is akin to ABC.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% trying ABC for starting values: N=500000 draws;
N=100; % This will take a long time... Drop to N=1000 for a quicker implementation

mu= -.1+.2.*rand(N,1);
beta= 0+.5.*rand(N,1);
alpha = 2.*rand(N,1);
sigmav = .2.*rand(N,1);
rho = -1+rand(N,1);
lamj =  .2.*rand(N,1);
muj = -.05+.1.*rand(N,1);
sigj = 2.*rand(N,1);
theta = [beta,alpha,sigmav,sigj,muj,lamj,rho];

 % Loop for calculating the criterion function at the evaluations 
 parfor i =1:N
 
     dist(i,1) = fun(theta(i,:)');
     i
 end
% Sorting them according to the smallest realization of the objective function 
tempN = [dist (1:1:N)' ];
B = sortrows(tempN,1);
tol=.01;    
% This can be set as the minimum, or an average over a bunch of "small"
% values for this tolerance. They give similar results for large N. 
level=floor(tol*size(B,1));
mean(theta(B(1:level,2),:))
std_theta=std(theta(B(1:level,2),:));
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

theta0=mean(theta(B(1:level,2),:))';


% Checking via optimization that this point is indeed a (local) minimum. 
lb = [0;0.001;0.001;0.001;-1;0.001;-1];
ub =[3;3.001;3.001;3.001;1;1;0];
options1 = optimoptions('fmincon','Display','iter','MaxIterations',1000,'MaxFunctionEvaluations',15000,'FunctionTolerance',1e-03,'Algorithm','sqp');
[theta_hat1,fval,exitflag] = fmincon(fun,theta0,[],[],[],[],lb,ub,[],options1);
 
lb_o = [theta_hat1(1);theta_hat1(2);theta_hat1(3);0.001;-1;0.001;-1];
ub_o =[theta_hat1(1);theta_hat1(2);theta_hat1(3);3.001;1;1;0];
[theta_hat2,fval,exitflag] = fmincon(fun,theta_hat1,[],[],[],[],lb_o,ub_o,[],options1);

%% Block Bootstrap approach to calculating standard errors. 
B = 999; % Number of bootstrap replications
len = 25; % Number of average block lengths. 
% Generating the bootstrap indices
[BSDATA, INDICES] = block_bootstrap(data(:,18),B,len);

% Loop to generate the new estimates for use in the bootstrap

for i=1:B
 i 
 index = INDICES(:,i);
 % Generating new draws to do the random grid search. For simplicitly,
 % we're only drawing these in a neighborhood of the point estimator to
 % reduce the number of random draws needed. This could also be done with a
 % larger number of draws and with new complete random draws as above.
N=1000;
beta= theta0(1)+std_theta(1).*(-1+2.*rand(N,1));
alpha = theta0(2)+std_theta(2).*(-1+2.*rand(N,1));
sigmav = theta0(3)+std_theta(3).*(-1+2.*rand(N,1));
sigj = theta0(4)+std_theta(4).*(-1+2.*rand(N,1));
muj = theta0(5)+std_theta(5).*(-1+2.*rand(N,1));
lamj =  theta0(6)+std_theta(6).*(-1+2.*rand(N,1));
rho = theta0(7)+std_theta(7).*(-1+2.*rand(N,1));

thetaBoot = [beta,alpha,sigmav,sigj,muj,lamj,rho];
 
% Generating the bootstrap versions of the observed data
rt_day = BSDATA(:,i);
rt_day = (rt_day - mean(rt_day));
rvt = data( index,11);
bvt = data( index,7);
jt = (rvt-bvt).*((rvt-bvt)>0);

% Estimating auxiliary parameters using the bootstrap data
 [parameters1, LL, ht, VCVrobust, VCV, scores, diagnostics] = tarch(rt_day, p, o, q, error_type,2,[.0000001;.01;.98;5]);

 % Calculating FUNC and auxiliary statistics based on bootstrap data
 FUNC = parameters1 - VCV*mean(scores)';
 
eta1_y = mean(sign(rt_day).*sqrt(jt));
eta2_y = mean((jt-mean(jt)).^2);
eta3_y = mean((jt-mean(jt)).*[0;(jt(1:end-1)-mean(jt))]);

Eta_y = [eta1_y;eta2_y;eta3_y]; 

% Just repeating the specifications from above.
M=4680; % number of 5 second intervals in a day
T=length(data)+1;
H=10; % Number of II reps.
nrepl = H;
D=12*5; % sampling frequency at 5 minutes (12*5*5 seconds)
beta_hat = parameters1;

% Note: all bootstrap draws must be based on the same set of random
% innovations W that were used to generate the initial estimators

% Specifying the function handle to optimize
 fun = @(t)IIFun(t,1, 0, 1, W, beta_hat,FUNC, Eta_y,T,M,H,D);
parfor j =1:N
     j
     distNew(j,1) = fun(thetaBoot(j,1:7)');
    
end
 
 tempN = [distNew (1:1:N)' ];
B = sortrows(tempN,1);

THETA_BOOT1(:,:,i) = thetaBoot(B(1,2),:)';

end
% The parameterization that has been used has kappa returns
% exp(eta). Therefore, need to transform via the log when reporting
% results.
temp=squeeze(THETA_BOOT1);
THETA_BOOT1(2,:,:)=log(THETA_BOOT1(2,:,:));
%% Saving results
%save('OXMAN_Jthetas_2yr_1000reps.mat','THETA_BOOT1','THETA_BOOT','theta0','theta_hat1','theta_hat2')
 
